#pragma once

#include "note.h"
#include "precisionevaluator.h"
#include "classicinputtype.h"

#include <memory>

struct Coordinates
{
    float x;
    float y;

    inline Coordinates operator+(const Coordinates& right) noexcept
    {
        return {right.x + x, right.y - y};
    }
};                              // MOVE TO OWN HEADER ^

class ClassicSprite;

class ClassicNote : public Note
{
public:

    enum class Grade
    {
        PERFECT,
           GOOD,
            BAD
    };

    enum class State
    {
          NONE,

        FLYING,
         DYING,
          DEAD
    };

    explicit ClassicNote(const std::vector<microsec>& intervals, microsec perfect_offset,
                         Action action, const Coordinates& coord);
    virtual ~ClassicNote() = default;

    virtual bool isActive(const microsec& music_offset) const override;
    virtual void update(const microsec &music_offset) override;
    virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const override;

    Grade input(ClassicInputType&& input_data);
    Action action() const;
    State state() const;

    void setState(State next_state);

    std::shared_ptr<ClassicSprite> sprite() const noexcept;
    void saveAppearanceTime(const microsec& offset);
    void setSprite(const std::shared_ptr<ClassicSprite>& sprite) noexcept;
    const Coordinates& getCoordinates() const noexcept;

private:
    const Coordinates _coordinates;
    const PrecisionEvaluator<Grade> _evaluator;
    const Action _action;
    State _state;

    std::shared_ptr<ClassicSprite> _sprite;
    microsec _appearance_time;
    float _trail_path_percent; //100% for sprite falling trajectory
};
